@@ -13,7 +13,7 @@ from oasys.widgets import gui as oasysgui
 from oasys.widgets import congruence
 from oasys.util.oasys_util import EmittingStream
 
-from orangecontrib.shadow4.util.shadow_objects import ShadowBeam
+from orangecontrib.shadow4.util.shadow4_objects import ShadowData
 from orangecontrib.shadow4.widgets.gui.ow_generic_element import GenericElement
 
 from shadow4.beamline.s4_beamline import S4Beamline
@@ -22,7 +22,7 @@ from shadow4.sources.source_geometrical.source_geometrical import SourceGeometri
 from syned.widget.widget_decorator import WidgetDecorator
 
 
-class OWUGeometrical(GenericElement, WidgetDecorator):
+class OWGeometrical(GenericElement, WidgetDecorator):
 
     name = "Geometrical Source"
     description = "Shadow Source: Geometrical Source"
@@ -32,8 +32,8 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
     inputs = []
     WidgetDecorator.append_syned_input_data(inputs)
 
-    outputs = [{"name":"Beam4",
-                "type":ShadowBeam,
+    outputs = [{"name":"Shadow Data",
+                "type":ShadowData,
                 "doc":"",}]
 
     number_of_rays = Setting(5000)
@@ -69,13 +69,14 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
 
     depth = Setting(0)
 
-    source_depth_y = Setting(0.2)
+    source_depth_y = Setting(0.002)
     sigma_y = Setting(0.001)
 
     photon_energy_distribution = Setting(0)
 
     units=Setting(0)
 
+    single_line_value = Setting(1000.0)
     single_line_value = Setting(1000.0)
     number_of_lines = Setting(0)
 
@@ -126,7 +127,7 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
 
 
     def __init__(self):
-        super().__init__(show_automatic_box=False)
+        super().__init__(show_automatic_box=False, has_footprint=False)
 
 
         self.runaction = widget.OWAction("Run Shadow4/Source", self)
@@ -144,7 +145,7 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
         button.setPalette(palette) # assign new palette
         button.setFixedHeight(45)
 
-        button = gui.button(button_box, self, "Reset Fields", callback=self.callResetSettings)
+        button = gui.button(button_box, self, "Reset Fields", callback=self.call_reset_settings)
         font = QFont(button.font())
         font.setItalic(True)
         button.setFont(font)
@@ -200,23 +201,23 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
 
         self.spatial_type_box_1 = oasysgui.widgetBox(spatial_type_box, "", addSpace=False, orientation="vertical")
 
-        self.le_rect_width = oasysgui.lineEdit(self.spatial_type_box_1, self, "rect_width", "Width", labelWidth=260,
+        self.le_rect_width = oasysgui.lineEdit(self.spatial_type_box_1, self, "rect_width", "Width X [m]", labelWidth=260,
                                                valueType=float, orientation="horizontal")
-        self.le_rect_height = oasysgui.lineEdit(self.spatial_type_box_1, self, "rect_height", "Height", labelWidth=260,
+        self.le_rect_height = oasysgui.lineEdit(self.spatial_type_box_1, self, "rect_height", "Height Z [m]", labelWidth=260,
                                                 valueType=float, orientation="horizontal")
 
         self.spatial_type_box_2 = oasysgui.widgetBox(spatial_type_box, "", addSpace=False, orientation="vertical")
 
-        self.le_ell_semiaxis_x = oasysgui.lineEdit(self.spatial_type_box_2, self, "ell_semiaxis_x", "Semi-Axis X",
+        self.le_ell_semiaxis_x = oasysgui.lineEdit(self.spatial_type_box_2, self, "ell_semiaxis_x", "Semi-Axis X [m]",
                                                    labelWidth=260, valueType=float, orientation="horizontal")
-        self.le_ell_semiaxis_z = oasysgui.lineEdit(self.spatial_type_box_2, self, "ell_semiaxis_z", "Semi-Axis Z",
+        self.le_ell_semiaxis_z = oasysgui.lineEdit(self.spatial_type_box_2, self, "ell_semiaxis_z", "Semi-Axis Z [m]",
                                                    labelWidth=260, valueType=float, orientation="horizontal")
 
         self.spatial_type_box_3 = oasysgui.widgetBox(spatial_type_box, "", addSpace=False, orientation="vertical")
 
-        self.le_gauss_sigma_x = oasysgui.lineEdit(self.spatial_type_box_3, self, "gauss_sigma_x", "Sigma X",
+        self.le_gauss_sigma_x = oasysgui.lineEdit(self.spatial_type_box_3, self, "gauss_sigma_x", "Sigma X [m]",
                                                   labelWidth=260, valueType=float, orientation="horizontal")
-        self.le_gauss_sigma_z = oasysgui.lineEdit(self.spatial_type_box_3, self, "gauss_sigma_z", "Sigma Z",
+        self.le_gauss_sigma_z = oasysgui.lineEdit(self.spatial_type_box_3, self, "gauss_sigma_z", "Sigma Z [m]",
                                                   labelWidth=260, valueType=float, orientation="horizontal")
 
         self.set_SpatialType()
@@ -224,7 +225,7 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
         angular_distribution_box = oasysgui.widgetBox(left_box_2, "Angular Distribution", addSpace=True,
                                                       orientation="vertical", height=260)
 
-        items = SourceGeometrical.angular_distribution_list() # ["Flat", "Uniform", "Gaussian", "Conical","Collimated"]
+        items = SourceGeometrical.angular_distribution_list() # ["Flat", "Uniform", "Gaussian", "Cone","Collimated"]
         gui.comboBox(angular_distribution_box, self, "angular_distribution", label="Angular Distribution",
                      labelWidth=355, items=items, orientation="horizontal", callback=self.set_AngularDistribution)
 
@@ -245,23 +246,24 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
         self.angular_distribution_box_2 = oasysgui.widgetBox(angular_distribution_box, "", addSpace=False,
                                                              orientation="vertical")
 
-        gui.comboBox(self.angular_distribution_box_2, self, "angular_distribution_limits",
-                     label="Angular Distribution Limits", labelWidth=355,
-                     items=["No Limits", "Horizontal", "Vertical", "Both"], orientation="horizontal",
-                     callback=self.set_AngularDistributionLimits)
+        if False: # No Gaussian limits for the moment (code kept just in case we want that in the future)
+            gui.comboBox(self.angular_distribution_box_2, self, "angular_distribution_limits",
+                         label="Angular Distribution Limits", labelWidth=355,
+                         items=["No Limits", "Horizontal", "Vertical", "Both"], orientation="horizontal",
+                         callback=self.set_AngularDistributionLimits)
 
-        self.le_horizontal_lim_x_plus = oasysgui.lineEdit(self.angular_distribution_box_2, self,
-                                                          "horizontal_lim_x_plus", "Horizontal Limit X(+) [rad]",
-                                                          labelWidth=260, valueType=float, orientation="horizontal")
-        self.le_horizontal_lim_x_minus = oasysgui.lineEdit(self.angular_distribution_box_2, self,
-                                                           "horizontal_lim_x_minus", "Horizontal Limit X(-) [rad]",
-                                                           labelWidth=260, valueType=float, orientation="horizontal")
-        self.le_vertical_lim_z_plus = oasysgui.lineEdit(self.angular_distribution_box_2, self, "vertical_lim_z_plus",
-                                                        "Vertical Limit Z(+) [rad]", labelWidth=260, valueType=float,
-                                                        orientation="horizontal")
-        self.le_vertical_lim_z_minus = oasysgui.lineEdit(self.angular_distribution_box_2, self, "vertical_lim_z_minus",
-                                                         "Vertical Limit Z(-) [rad]", labelWidth=260, valueType=float,
-                                                         orientation="horizontal")
+            self.le_horizontal_lim_x_plus = oasysgui.lineEdit(self.angular_distribution_box_2, self,
+                                                              "horizontal_lim_x_plus", "Horizontal Limit X(+) [rad]",
+                                                              labelWidth=260, valueType=float, orientation="horizontal")
+            self.le_horizontal_lim_x_minus = oasysgui.lineEdit(self.angular_distribution_box_2, self,
+                                                               "horizontal_lim_x_minus", "Horizontal Limit X(-) [rad]",
+                                                               labelWidth=260, valueType=float, orientation="horizontal")
+            self.le_vertical_lim_z_plus = oasysgui.lineEdit(self.angular_distribution_box_2, self, "vertical_lim_z_plus",
+                                                            "Vertical Limit Z(+) [rad]", labelWidth=260, valueType=float,
+                                                            orientation="horizontal")
+            self.le_vertical_lim_z_minus = oasysgui.lineEdit(self.angular_distribution_box_2, self, "vertical_lim_z_minus",
+                                                             "Vertical Limit Z(-) [rad]", labelWidth=260, valueType=float,
+                                                             orientation="horizontal")
 
         oasysgui.lineEdit(self.angular_distribution_box_2, self, "horizontal_sigma_x", "Horizontal Sigma (X) [rad]",
                           labelWidth=260, valueType=float, orientation="horizontal")
@@ -289,12 +291,12 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
 
         self.depth_box_1 = oasysgui.widgetBox(depth_box, "", addSpace=False, orientation="vertical")
 
-        self.le_source_depth_y = oasysgui.lineEdit(self.depth_box_1, self, "source_depth_y", "Source Depth (Y)",
+        self.le_source_depth_y = oasysgui.lineEdit(self.depth_box_1, self, "source_depth_y", "Source Depth (Y) [m]",
                                                    labelWidth=260, valueType=float, orientation="horizontal")
 
         self.depth_box_2 = oasysgui.widgetBox(depth_box, "", addSpace=False, orientation="vertical")
 
-        self.le_sigma_y = oasysgui.lineEdit(self.depth_box_2, self, "sigma_y", "Sigma Y", labelWidth=260,
+        self.le_sigma_y = oasysgui.lineEdit(self.depth_box_2, self, "sigma_y", "Sigma Y [m]", labelWidth=260,
                                             valueType=float, orientation="horizontal")
 
         self.set_Depth()
@@ -477,8 +479,8 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
     def is_scanning_enabled(self):
         return True
 
-    def callResetSettings(self):
-        super().callResetSettings()
+    def call_reset_settings(self):
+        super().call_reset_settings()
 
         self.set_Sampling()
         self.set_SpatialType()
@@ -496,10 +498,11 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
         self.spatial_type_box_3.setVisible(self.spatial_type == 3)
 
     def set_AngularDistributionLimits(self):
-        self.le_horizontal_lim_x_plus.setEnabled(self.angular_distribution_limits != 0 and self.angular_distribution_limits != 2)
-        self.le_horizontal_lim_x_minus.setEnabled(self.angular_distribution_limits != 0 and self.angular_distribution_limits != 2)
-        self.le_vertical_lim_z_plus.setEnabled(self.angular_distribution_limits != 0 and self.angular_distribution_limits != 1)
-        self.le_vertical_lim_z_minus.setEnabled(self.angular_distribution_limits != 0 and self.angular_distribution_limits != 1)
+        if False:
+            self.le_horizontal_lim_x_plus.setEnabled(self.angular_distribution_limits != 0 and self.angular_distribution_limits != 2)
+            self.le_horizontal_lim_x_minus.setEnabled(self.angular_distribution_limits != 0 and self.angular_distribution_limits != 2)
+            self.le_vertical_lim_z_plus.setEnabled(self.angular_distribution_limits != 0 and self.angular_distribution_limits != 1)
+            self.le_vertical_lim_z_minus.setEnabled(self.angular_distribution_limits != 0 and self.angular_distribution_limits != 1)
 
     def set_AngularDistribution(self):
         self.angular_distribution_box_1.setVisible(self.angular_distribution == 0 or self.angular_distribution == 1)
@@ -607,6 +610,14 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
                                          sigma_v=self.gauss_sigma_z,
                                          )
 
+        if self.depth == 0:
+            gs.set_depth_distribution_off()
+        elif self.depth == 1:
+            gs.set_depth_distribution_uniform(self.source_depth_y)
+        elif self.depth == 2:
+            gs.set_depth_distribution_gaussian(self.sigma_y)
+
+
         if self.angular_distribution == 0: # flat
             gs.set_angular_distribution_flat(hdiv1=-self.horizontal_div_x_minus,
                                              hdiv2=self.horizontal_div_x_plus,
@@ -620,16 +631,13 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
                                                 vdiv2=self.vertical_div_z_plus,
                                                )
         elif self.angular_distribution == 2:  # Gaussian
-            if self.angular_distribution_limits != 0:
-                raise Exception("Not yet implemented")
-
-            gs.set_angular_distribution_gaussian(sigdix=self.gauss_sigma_x,
-                                                 sigdiz=self.gauss_sigma_z,
+            gs.set_angular_distribution_gaussian(sigdix=self.horizontal_sigma_x,
+                                                 sigdiz=self.vertical_sigma_z,
                                                  )
         elif self.angular_distribution == 3:  # cone
-            gs.set_angular_distribution_conical(cone_max=self.cone_external_half_aperture,
-                                                cone_min=self.cone_internal_half_aperture,
-                                                )
+            gs.set_angular_distribution_cone(cone_max=self.cone_external_half_aperture,
+                                             cone_min=self.cone_internal_half_aperture,
+                                             )
 
         elif self.angular_distribution == 4:  # Zero (collimated) - New in shadow4
             gs.set_angular_distribution_collimated()
@@ -688,57 +696,54 @@ class OWUGeometrical(GenericElement, WidgetDecorator):
 
     def run_shadow4(self):
 
-        sys.stdout = EmittingStream(textWritten=self.writeStdOut)
+        sys.stdout = EmittingStream(textWritten=self._write_stdout)
 
-        self.set_PlotQuality()
+        self._set_plot_quality()
 
         self.progressBarInit()
 
         light_source = self.get_lightsource()
 
+        # script
+        script = light_source.to_python_code()
+        script += "\n\n# test plot\nfrom srxraylib.plot.gol import plot_scatter"
+        script += "\nrays = beam.get_rays()"
+        script += "\nplot_scatter(1e6 * rays[:, 0], 1e6 * rays[:, 2], title='(X,Z) in microns')"
+        self.shadow4_script.set_code(script)
+
+
+        print(light_source.info())
+
+        print(light_source.get_info())
+
         self.progressBarSet(5)
 
         # run shadow4
-
         t00 = time.time()
         # beam = light_source.get_beam(NRAYS=self.number_of_rays, SEED=self.seed)
-        beam = light_source.get_beam()
+        output_beam = light_source.get_beam()
         t11 = time.time() - t00
         print(">>>> time for %d rays: %f s, %f min, " % (self.number_of_rays, t11, t11 / 60))
 
         #
         # beam plots
         #
-        beamline = S4Beamline(light_source=light_source)
+        self._plot_results(output_beam, None, progressBarValue=80)
 
-        BEAM = ShadowBeam(beam=beam, oe_number=0, number_of_rays=self.number_of_rays, beamline=beamline)
-        self.plot_results(BEAM, progressBarValue=80)
 
-        #
-        # script
-        #
-        script = light_source.to_python_code()
-        # script += "\n\n\n# run shadow4"
-        # script += "\nbeam = light_source.get_beam(N=%d, ISTAR1=%d)" % \
-        #           (self.number_of_rays, self.seed)
-
-        script += "\n\n# test plot\nfrom srxraylib.plot.gol import plot_scatter"
-        script += "\nrays = beam.get_rays()"
-        script += "\nplot_scatter(1e6 * rays[:, 0], 1e6 * rays[:, 2], title='(X,Z) in microns')"
-
-        self.shadow4_script.set_code(script)
 
         self.progressBarFinished()
 
         #
         # send beam
         #
-        self.send("Beam4", BEAM)
-
+        self.send("Shadow Data", ShadowData(beam=output_beam,
+                                           number_of_rays=self.number_of_rays,
+                                           beamline=S4Beamline(light_source=light_source)))
 if __name__ == "__main__":
     from PyQt5.QtWidgets import QApplication
     a = QApplication(sys.argv)
-    ow = OWUGeometrical()
+    ow = OWGeometrical()
     ow.show()
     a.exec_()
     ow.saveSettings()
