@@ -3,7 +3,7 @@ import time
 
 from orangecontrib.shadow4.widgets.gui.ow_electron_beam import OWElectronBeam
 from orangecontrib.shadow4.widgets.gui.plots import plot_data1D
-from orangecontrib.shadow4.util.shadow_objects import ShadowBeam
+from orangecontrib.shadow4.util.shadow4_objects import ShadowData
 
 from oasys.widgets import gui as oasysgui
 
@@ -32,8 +32,8 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
     inputs = []
     WidgetDecorator.append_syned_input_data(inputs)
 
-    outputs = [{"name":"Beam4",
-                "type":ShadowBeam,
+    outputs = [{"name":"Shadow Data",
+                "type":ShadowData,
                 "doc":"",}]
 
     magnetic_field_source = Setting(1)
@@ -51,9 +51,19 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
 
     e_min = Setting(0.4)
     e_max = Setting(0.4)
-    number_of_rays = Setting(500)
+    number_of_rays = Setting(15000)
     seed = Setting(5676561)
 
+    # number of points in advanced parameters
+    ng_e = Setting(101)
+    ng_j = Setting(501)
+    psi_interval_number_of_points = Setting(101)
+
+    epsi_dx = Setting(0.0)
+    epsi_dz = Setting(0.0)
+
+    flag_interpolation = Setting(2)
+
     plot_wiggler_graph = 1
 
     beam_out = None
@@ -67,65 +77,76 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
         # wiggler parameters box
         left_box_3 = oasysgui.widgetBox(tab_wiggler, "Wiggler Parameters", addSpace=False, orientation="vertical", height=200)
 
-        orangegui.comboBox(left_box_3, self, "magnetic_field_source", label="Type", items=["conventional/sinusoidal", "B from file (y [m], Bz [T])", "B from harmonics"], callback=self.set_visibility, labelWidth=220, orientation="horizontal")
+        orangegui.comboBox(left_box_3, self, "magnetic_field_source", tooltip="magnetic_field_source", label="Type", items=["conventional/sinusoidal", "B from file (y [m], Bz [T])", "B from harmonics"], callback=self.set_visibility, labelWidth=220, orientation="horizontal")
 
-        oasysgui.lineEdit(left_box_3, self, "number_of_periods", "Number of Periods", labelWidth=260, tooltip="Number of Periods", valueType=int, orientation="horizontal")
+        oasysgui.lineEdit(left_box_3, self, "number_of_periods", "Number of Periods", tooltip="number_of_periods",labelWidth=260, valueType=int, orientation="horizontal")
 
         self.conventional_sinusoidal_box = oasysgui.widgetBox(left_box_3, "", addSpace=False, orientation="vertical")
 
-        oasysgui.lineEdit(self.conventional_sinusoidal_box, self, "k_value", "K value", labelWidth=260, tooltip="K value", valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(self.conventional_sinusoidal_box, self, "id_period", "ID period [m]", labelWidth=260, tooltip="ID period [m]", valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(self.conventional_sinusoidal_box, self, "k_value", "K value", labelWidth=260, tooltip="k_value", valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(self.conventional_sinusoidal_box, self, "id_period", "ID period [m]", labelWidth=260, tooltip="id_period", valueType=float, orientation="horizontal")
 
         self.b_from_file_box = oasysgui.widgetBox(left_box_3, "", addSpace=False, orientation="vertical")
 
         file_box = oasysgui.widgetBox(self.b_from_file_box, "", addSpace=True, orientation="horizontal", height=25)
 
-        self.le_file_with_b_vs_y = oasysgui.lineEdit(file_box, self, "file_with_b_vs_y", "File/Url with B vs Y", labelWidth=150, tooltip="File/Url with B vs Y", valueType=str, orientation="horizontal")
+        self.le_file_with_b_vs_y = oasysgui.lineEdit(file_box, self, "file_with_b_vs_y", "File/Url with B vs Y", labelWidth=150, tooltip="file_with_b_vs_y", valueType=str, orientation="horizontal")
 
         orangegui.button(file_box, self, "...", callback=self.select_file_with_B_vs_Y)
 
         self.b_from_harmonics_box = oasysgui.widgetBox(left_box_3, "", addSpace=False, orientation="vertical")
 
-        oasysgui.lineEdit(self.b_from_harmonics_box, self, "id_period", "ID period [m]", labelWidth=260, tooltip="ID period [m]", valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(self.b_from_harmonics_box, self, "id_period", "ID period [m]", labelWidth=260, tooltip="id_period", valueType=float, orientation="horizontal")
 
         file_box = oasysgui.widgetBox(self.b_from_harmonics_box, "", addSpace=True, orientation="horizontal", height=25)
 
-        self.le_file_with_harmonics = oasysgui.lineEdit(file_box, self, "file_with_harmonics", "File/Url with harmonics", labelWidth=150, tooltip="File/Url with harmonics", valueType=str, orientation="horizontal")
+        self.le_file_with_harmonics = oasysgui.lineEdit(file_box, self, "file_with_harmonics", "File/Url with harmonics", labelWidth=150, tooltip="file_with_harmonics", valueType=str, orientation="horizontal")
 
         orangegui.button(file_box, self, "...", callback=self.select_file_with_harmonics)
 
-
         # Electron initial conditions Box
         left_box_10 = oasysgui.widgetBox(tab_wiggler, "Electron Initial Condition", addSpace=False, orientation="vertical", height=200)
 
-
-        orangegui.comboBox(left_box_10, self, "shift_betax_flag", label="Shift Transversal Velocity", items=["No shift", "Half excursion", "Minimum", "Maximum", "Value at zero", "User value"], callback=self.set_shift_beta_X_flag, labelWidth=260, orientation="horizontal")
+        orangegui.comboBox(left_box_10, self, "shift_betax_flag", tooltip="shift_betax_flag", label="Shift Transversal Velocity", items=["No shift", "Half excursion", "Minimum", "Maximum", "Value at zero", "User value"], callback=self.set_shift_beta_X_flag, labelWidth=260, orientation="horizontal")
         self.shift_betax_value_box = oasysgui.widgetBox(left_box_10, "", addSpace=False, orientation="vertical", height=25)
         self.shift_betax_value_box_hidden = oasysgui.widgetBox(left_box_10, "", addSpace=False, orientation="vertical", height=25)
-        oasysgui.lineEdit(self.shift_betax_value_box, self, "shift_betax_value", "Value", labelWidth=260, valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(self.shift_betax_value_box, self, "shift_betax_value", "Value", tooltip="shift_betax_value", labelWidth=260, valueType=float, orientation="horizontal")
 
-        orangegui.comboBox(left_box_10, self, "shift_x_flag", label="Shift Transversal Coordinate", items=["No shift", "Half excursion", "Minimum", "Maximum", "Value at zero", "User value"], callback=self.set_shift_X_flag, labelWidth=260, orientation="horizontal")
+        orangegui.comboBox(left_box_10, self, "shift_x_flag", tooltip="shift_x_flag", label="Shift Transversal Coordinate", items=["No shift", "Half excursion", "Minimum", "Maximum", "Value at zero", "User value"], callback=self.set_shift_X_flag, labelWidth=260, orientation="horizontal")
         self.shift_x_value_box = oasysgui.widgetBox(left_box_10, "", addSpace=False, orientation="vertical", height=25)
         self.shift_x_value_box_hidden = oasysgui.widgetBox(left_box_10, "", addSpace=False, orientation="vertical", height=25)
-        oasysgui.lineEdit(self.shift_x_value_box, self, "shift_x_value", "Value [m]", labelWidth=260, valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(self.shift_x_value_box, self, "shift_x_value", "Value [m]", tooltip="shift_x_value", labelWidth=260, valueType=float, orientation="horizontal")
 
         self.set_shift_X_flag()
         self.set_shift_beta_X_flag()
 
-
-
         # Calculation Box
         left_box_11 = oasysgui.widgetBox(tab_wiggler, "Sampling rays", addSpace=False, orientation="vertical", height=200)
 
-        oasysgui.lineEdit(left_box_11, self, "e_min", "Min photon energy [eV]", labelWidth=260, valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(left_box_11, self, "e_max", "Max photon energy [eV]", labelWidth=260, valueType=float, orientation="horizontal")
-        oasysgui.lineEdit(left_box_11, self, "number_of_rays", "Number of rays", labelWidth=260, valueType=int, orientation="horizontal")
-        oasysgui.lineEdit(left_box_11, self, "seed", "Seed", tooltip="Seed (0=clock)", labelWidth=250, valueType=int, orientation="horizontal")
+        oasysgui.lineEdit(left_box_11, self, "e_min", "Min photon energy [eV]", tooltip="e_min", labelWidth=260, valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(left_box_11, self, "e_max", "Max photon energy [eV]", tooltip="e_max", labelWidth=260, valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(left_box_11, self, "number_of_rays", "Number of rays", tooltip="number_of_rays", labelWidth=260, valueType=int, orientation="horizontal")
+        oasysgui.lineEdit(left_box_11, self, "seed", "Seed", tooltip="seed", labelWidth=250, valueType=int, orientation="horizontal")
 
         self.set_shift_X_flag()
         self.set_shift_beta_X_flag()
 
 
+        # wiggler adv settings
+        tab_advanced = oasysgui.createTabPage(self.tabs_control_area, "Advanced Setting")
+        left_box_adv = oasysgui.widgetBox(tab_advanced, "Advanced settings", addSpace=False, orientation="vertical", height=200)
+        oasysgui.lineEdit(left_box_adv, self, "ng_e", "Number of Points in energy scan", labelWidth=260, tooltip="ng_e", valueType=int, orientation="horizontal")
+        oasysgui.lineEdit(left_box_adv, self, "ng_j", "Number of Points in e trajectory (per period)", labelWidth=280, tooltip="ng_j", valueType=int, orientation="horizontal")
+        oasysgui.lineEdit(left_box_adv, self, "psi_interval_number_of_points", "Number of Points in sampling vertical angle", labelWidth=280, tooltip="psi_interval_number_of_points", valueType=int, orientation="horizontal")
+
+
+        oasysgui.lineEdit(left_box_adv, self, "epsi_dx", "Distance from waist X [m]", labelWidth=260, tooltip="epsi_dx", valueType=float, orientation="horizontal")
+        oasysgui.lineEdit(left_box_adv, self, "epsi_dz", "Distance from waist Z [m]", labelWidth=260, tooltip="epsi_dz", valueType=float, orientation="horizontal")
+
+        orangegui.comboBox(left_box_adv, self, "flag_interpolation", tooltip="flag_interpolation", label="Sample psi via interpolation",
+                           items=["No (accurate, exact Bessel)", "Yes (good for mono or quasi monochromatic)", "No (accurate, approx Bessel)"], labelWidth=260, orientation="horizontal")
+
+
         # wiggler plots
         self.add_specific_wiggler_plots()
 
@@ -133,9 +154,7 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
 
         orangegui.rubber(self.controlArea)
 
-
     def add_specific_wiggler_plots(self):
-
         wiggler_plot_tab = oasysgui.widgetBox(self.main_tabs, addToLayout=0, margin=4)
 
         self.main_tabs.insertTab(1, wiggler_plot_tab, "Wiggler Plots")
@@ -202,9 +221,6 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
     def get_lightsource(self):
         # syned
         electron_beam = self.get_electron_beam()
-        print("\n\n>>>>>> ElectronBeam info: ", electron_beam.info(), type(electron_beam))
-
-        ng_j = 501 # trajectory points
 
         if self.type_of_properties == 3:
             flag_emittance = 0
@@ -220,20 +236,24 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
                     number_of_periods        = self.number_of_periods, # syned Wiggler pars: useful if magnetic_field_periodic=1
                     emin                     = self.e_min,     # Photon energy scan from energy (in eV)
                     emax                     = self.e_max,     # Photon energy scan to energy (in eV)
-                    ng_e                     = 100,            # Photon energy scan number of points
-                    ng_j                     = ng_j ,          # Number of points in electron trajectory (per period) for internal calculation only
+                    ng_e                     = self.ng_e,      # Photon energy scan number of points
+                    ng_j                     = self.ng_j ,     # Number of points in electron trajectory (per period) for internal calculation only
+                    psi_interval_number_of_points = self.psi_interval_number_of_points,
+                    flag_interpolation       = self.flag_interpolation,  # Use intyerpolation to sample psi (0=No, 1=Yes)
                     flag_emittance           = flag_emittance, # Use emittance (0=No, 1=Yes)
                     shift_x_flag             = 0,
                     shift_x_value            = 0.0,
                     shift_betax_flag         = 0,
                     shift_betax_value        = 0.0,
+                    epsi_dx                  = self.epsi_dx,
+                    epsi_dz                  = self.epsi_dz,
                     )
 
         elif self.magnetic_field_source == 1:
             if self.e_min == self.e_max:
                 ng_e = 1
             else:
-                ng_e = 10
+                ng_e = self.ng_e
 
             sourcewiggler = S4Wiggler(
                 magnetic_field_periodic   = 0,
@@ -241,7 +261,9 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
                 emin                      = self.e_min,
                 emax                      = self.e_max,
                 ng_e                      = ng_e,
-                ng_j                      = ng_j,
+                ng_j                      = self.ng_j,
+                psi_interval_number_of_points=self.psi_interval_number_of_points,
+                flag_interpolation=self.flag_interpolation,  # Use intyerpolation to sample psi (0=No, 1=Yes)
                 flag_emittance            = flag_emittance,  # Use emittance (0=No, 1=Yes)
                 shift_x_flag              = 4,
                 shift_x_value             = 0.0,
@@ -265,8 +287,7 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
                         shift_betax_flag=self.shift_betax_flag,
                         shift_betax_value=self.shift_betax_value)
 
-        print(">>>>>> \n\n S4Wiggler info: ", sourcewiggler.info())
-
+        print(">>>>>> \n\n S4Wiggler get_info: \n", sourcewiggler.get_info())
 
         # S4WigglerLightSource
         lightsource = S4WigglerLightSource(name='wiggler',
@@ -275,42 +296,20 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
                                            nrays=self.number_of_rays,
                                            seed=self.seed)
 
-        print("\n\n>>>>>> S4WigglerLightSource info: ", lightsource.info())
+        print("\n\n>>>>>> S4WigglerLightSource info: \n", lightsource.info())
 
         return lightsource
 
 
     def run_shadow4(self):
+        sys.stdout = EmittingStream(textWritten=self._write_stdout)
 
-        sys.stdout = EmittingStream(textWritten=self.writeStdOut)
-
-        self.set_PlotQuality()
+        self._set_plot_quality()
 
         self.progressBarInit()
 
         light_source = self.get_lightsource()
 
-        self.progressBarSet(5)
-        #
-        # run shadow4
-        #
-        t00 = time.time()
-        print(">>>> starting calculation...")
-        beam = light_source.get_beam()
-        photon_energy, flux, spectral_power = light_source.calculate_spectrum()
-        t11 = time.time() - t00
-        print(">>>> time for %d rays: %f s, %f min, " % (self.number_of_rays, t11, t11 / 60))
-
-
-        #
-        # plots
-        #
-        beamline = S4Beamline(light_source=light_source)
-        BEAM = ShadowBeam(beam=beam, oe_number=0, number_of_rays=self.number_of_rays, beamline=beamline)
-        self.plot_results(BEAM, progressBarValue=80)
-        self.refresh_specific_wiggler_plots(light_source, photon_energy, flux, spectral_power)
-
-
         #
         # script
         #
@@ -319,28 +318,36 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
         script += "\nrays = beam.get_rays()"
         script += "\nplot_scatter(1e6 * rays[:, 0], 1e6 * rays[:, 2], title='(X,Z) in microns')"
 
-#         script_template = """
-#
-# # run shadow4
-# beam = light_source.get_beam(NRAYS={NRAYS}, SEED={SEED})
-# photon_energy, flux, spectral_power = light_source.calculate_spectrum()"""
-#         script_dict = {
-#             "NRAYS": self.number_of_rays,
-#             "SEED": self.seed,
-#         }
-#
-#         script += script_template.format_map(script_dict)
-
-
         self.shadow4_script.set_code(script)
 
+        #
+        # run shadow4
+        #
+        self.progressBarSet(5)
+
+
+        self.progressBarSet(10)
+        t00 = time.time()
+        print(">>>> starting shadow calculation...")
+        output_beam = light_source.get_beam()
+        photon_energy, flux, spectral_power = light_source.calculate_spectrum()
+        t11 = time.time() - t00
+        print(">>>> time for %d rays: %f s, %f min, " % (self.number_of_rays, t11, t11 / 60))
+
+        #
+        # plots
+        #
+        self._plot_results(output_beam, None, progressBarValue=80)
+        self.refresh_specific_wiggler_plots(light_source, photon_energy, flux, spectral_power)
+
         self.progressBarFinished()
 
         #
         # send beam
         #
-        self.send("Beam4", BEAM)
-
+        self.send("Shadow Data", ShadowData(beam=output_beam,
+                                           number_of_rays=self.number_of_rays,
+                                           beamline=S4Beamline(light_source=light_source)))
 
     def refresh_specific_wiggler_plots(self, lightsource=None, e=None, f=None, w=None):
 
@@ -382,7 +389,7 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
         self.wiggler_tab[wiggler_plot_slot_index].layout().addWidget(plot_widget_id)
 
     def receive_syned_data(self, data):
-        sys.stdout = EmittingStream(textWritten=self.writeStdOut)
+        sys.stdout = EmittingStream(textWritten=self._write_stdout)
         if data is not None:
             if isinstance(data, Beamline):
                 if not data.get_light_source() is None:
@@ -399,7 +406,6 @@ class OWWiggler(OWElectronBeam, WidgetDecorator):
                         self.k_value = w.K_vertical()
 
                         self.populate_fields_from_electron_beam(light_source.get_electron_beam())
-
                     else:
                         raise ValueError("Syned light source not congruent")
                 else:
